/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2011, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//AwakeFileSession : Awake File main class
//
//Copyright (c) Kawan Softwares S.A.S, 2011
//
//Last Updates: 
// 22/09/09 14:00 NDP - Creation
// 22/09/09 16:20 NDP - Add ServerCallerNew.getHostFileSize()
// 26/02/10 11:20 NDP - Add setAwakeProgressManager() (replace setFilesLength())
// 26/02/10 12:10 NDP - AwakeFileSession: No more pass of HttpNetworkParameters in constructor
// 02/03/10 16:20 NDP - AwakeFileSession: Do not throw illegalArgumentValue on constructors
// 22/03/10 17:55 NDP - Do not transmit username/password before checking if we must be in https
// 25/03/10 18:00 NDP - Remote call takes only one parameter for class+method name
// 26/03/10 18:45 NDP - Clean comments
// 02/07/10 21:10 NDP - Add downloadUrl(URL url, File file) 
// 03/07/10 18:50 NDP - Add setHttpNetworkParameters() for downloadUrl
// 04/07/10 18:50 NDP - Remove setHttpNetworkParameters() method
// 04/07/10 18:50 NDP - Now all AwakeFileSession constructors must pass a HttpNetworkParameters instance
// 04/07/10 18:50 NDP - Do not pass HttpNetworkParameters instance to username() method
// 05/07/10 11:40 NDP - Add getHttpNetworkParameters()
// 15/07/10 19:25 NDP - AwakeFileSession: Remove dead code in uploadFile of test of remoteFilePath == null
// 27/08/10 20:10 NDP - use now JsonList to transport list of strings
// 28/08/10 18:30 NDP - listFilesOrDirectory() gets content from a string instead of a file
// 28/08/10 19:15 NDP - AwakeFileSession: Add DELETE_RESULT_FILE to delete temp file
// 15/09/10 14:00 NDP - AwakeFileSession: new constructors with/without proxy & setHttpProtocolParameters
// 20/09/10 20:05 NDP - New header comments & username returns void
// 21/09/10 16:35 NDP - Port for new HttpClient 4.x
// 24/09/10 12:45 NDP - AwakeFileSession.call(): String returned by Awake File Server is now base64
// 30/09/10 12:30 NDP - Version that throws Exceptions
// 27/11/10 19:10 NDP - AwakeFileSession: add getUrl()
// 29/12/10 17:30 NDP - AwakeFileSession.call(): remove a silly println
// 16/01/11 20:10 NDP : AwakeFileSession.call: apply HtmlConverter.toHtml() to remote file name before servlet call
// 16/01/11 20:25 NDP : AwakeFileSession.call: apply HtmlConverter.fromHtml() to local file after download
// 17/01/11 17:20 NDP : AwakeFileSession.call: apply HtmlConverter.toHtml() to username & password
// 17/01/11 17:20 NDP : AwakeFileSession.call: NO MORE apply HtmlConverter.toHtml() to username & password
// 17/01/11 17:45 NDP : AwakeFileSession.call: this.login = username done before conversion
// 13/03/10 15:45 NDP : AwakeFileSession: no more setter for HttpProtocolParameters (passed in constructor)
// 12/04/12 18:15 NDP : AwakeFileSession: clean trap and re-throw of remote exceptions
// 03/06/11 20:00 NDP : AwakeFileSession: Refactor login(): returns a boolean
// 08/06/11 19:30 NDP : AwakeFileSession: no more throw of AwakeRemoteException on apis, except for call()
// 19/06/11 16:00 NDP : AwakeFileSession: add getUsername() and getAwakeProgressManager()
// 20/06/11 12:05 NDP : AwakeFileSession: login & username are now passed to constructor
// 20/06/11 14:40 NDP : AwakeFileSession: getStatusCode() renamed to getHttpStatusCode()
// 23/06/11 13:25 NDP : AwakeFileSession: username & password may be null: for call() and downloadUrl() usage
// 23/06/11 16:00 NDP : AwakeFileSession: comments
// 15/07/11 11:50 NDP : AwakeFileSession: add clone() method
// 20/07/11 18:10 NDP : AwakeFileSession: add getUrlContent(URL url) 
// 22/07/11 18:20 NDP : AwakeFileSession: adapt URL access code to new AwakeUrl
// 22/07/11 18:20 NDP : UrlDownloaderEngine: adapt URL access code to new AwakeUrl
// 30/09/11 15:25 NDP : AwakeFileSession:  authenticationToken = StringUtils.left(theToken, Parameter.TOKEN_LEFT_SIZE);
// 01/10/11 21:25 NDP : AwakeFileSession: remove package name from class name in call()
// 07/10/11 20:05 NDP : AwakeFileSession: NO! Package name must not be removed from class name in call()
// 27/10/11 20:00 NDP : AwakeFileSession: null parameters are refused for call()
// 30/11/11 19:35 NDP : AwakeFileSession: delete receiveFile if there is an Exception
// 07/12/11 18:10 NDP : AwakeFileSession: clean getRemoteFilesLength(List<String> remoteFilesPath) javadoc
// 07/12/11 18:10 NDP : AwakeFileSession: clean listRemoteXxxx javadoc
// 08/12/11 19:05 NDP : AwakeFileSession: add callHtmlEncoded()
// 08/12/11 19:10 NDP : AwakeFileSession: no more encode for parameter values, except for old call()
// 10/12/11 15:10 NDP : AwakeFileSession: add setUseBase64EncodingForCall()
// 10/12/11 21:05 NDP : AwakeFileSession: callBase64Encoded: no more username encode
// 12/12/11 17:30 NDP : AwakeFileSession: call(): no more base64 trap exception on Html decode 

package org.awakefw.file.api.client;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.ConnectException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.List;
import java.util.Vector;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.message.BasicNameValuePair;
import org.awakefw.commons.api.client.AwakeProgressManager;
import org.awakefw.commons.api.client.AwakeRemoteException;
import org.awakefw.commons.api.client.AwakeSecurityException;
import org.awakefw.commons.api.client.HttpProtocolParameters;
import org.awakefw.commons.api.client.HttpProxy;
import org.awakefw.commons.api.client.InvalidLoginException;
import org.awakefw.commons.server.util.AwakeLogger;
import org.awakefw.file.api.util.AwakeDebug;
import org.awakefw.file.api.util.HtmlConverter;
import org.awakefw.file.http.HttpTransfer;
import org.awakefw.file.http.HttpTransferOne;
import org.awakefw.file.http.exception.MyIOInterruptedException;
import org.awakefw.file.json.JsonClientBuilder;
import org.awakefw.file.json.ListHolder;
import org.awakefw.file.util.KeepTempFilePolicyParms;
import org.awakefw.file.util.StringUtil;
import org.awakefw.file.util.Tag;
import org.awakefw.file.util.parms.Action;
import org.awakefw.file.util.parms.Parameter;
import org.awakefw.file.util.parms.ReturnCode;
import org.awakefw.file.version.AwakeFileVersion;

/**
 * Main class for executing from a Client (typically a PC) all sorts of operations on remote files or
 * remote classes through a Http or Https session.
 * <p>
 * Class includes methods:
 * <ul>
 * <li>Get remote files size.</li>
 * <li>Delete remote files.</li>
 * <li>Create a remote directory.</li>
 * <li>Get a remote directory list.</li>
 * <li>Upload files.</li>
 * <li>Download files.</li>
 * <li>Call remote Java methods.</li>
 * </ul>
 * Example: <blockquote>
 * 
 * <pre>
 * // Create an AwakeFile instance and username on the remote server
 * // using the URL of the path to the AwakeFileManager Servlet
 * String url = &quot;https://www.acme.org/AwakeFileManager&quot;;
 * &nbsp;
 * // The login info for strong authentication on server side:
 * String username = &quot;myUsername&quot;;
 * char[] password = { 'm', 'y', 'P', 'a', 's', 's', 'w', 'o', 'r', 'd' };
 * &nbsp;
 * AwakeFileSession awakeFileSession = new AwakeFileSession(url, username,
 * 	password);
 * &nbsp;
 * // OK: upload a file
 * awakeFileSession.uploadFile(new File(&quot;c:\\myFile.txt&quot;), &quot;myFile.txt&quot;);
 * </pre>
 * 
 * </blockquote>
 * <p>
 * Communication via an (authenticating) proxy server is done using a
 * {@link org.awakefw.commons.api.client.HttpProxy} instance: <blockquote>
 * 
 * <pre>
 * HttpProxy httpProxy = new HttpProxy(&quot;myproxyhost&quot;, 8080);
 * String url = &quot;https://www.acme.org/AwakeFileManager&quot;;
 * &nbsp;
 * // The login info for strong authentication on server side:
 * String username = &quot;myUsername&quot;;
 * char[] password = { 'm', 'y', 'P', 'a', 's', 's', 'w', 'o', 'r', 'd' };
 * &nbsp;
 * AwakeFileSession awakeFileSession = new AwakeFileSession(url, username,
 * 	password, httpProxy);
 * &nbsp;
 * // Etc.
 * 
 </pre>
 * 
 * </blockquote>
 * <p>
 * All long operations that need to be run on a separated thread may be followed
 * in Swing using a Java Progress Bar or Progress Monitor: just implement the
 * {@link org.awakefw.commons.api.client.AwakeProgressManager} interface.
 * <p>
 * 
 * @see org.awakefw.commons.api.client.HttpProxy
 * @see org.awakefw.commons.api.client.AwakeProgressManager
 * 
 * @author Nicolas de Pomereu
 * @since 1.0
 */

public final class AwakeFileSession implements Cloneable {
    /** For debug info */
    private static boolean DEBUG = AwakeDebug.isSet(AwakeFileSession.class);

    /** Defines if we must use base64 or html encode when using call() */
    private static boolean USE_HTML_ENCODING = true;
    
    /** The url to use to connectet to the Awake File Server */
    private String url = null;

    /**
     * The username is stored in static memory to be passed to upload file
     * servlet
     */
    private String username = null;

    /**
     * Token is stored in static to be available during all session and contains
     * SHA-1(userId + ServerClientLogin.SECRET_FOR_LOGIN) computed by server.
     * Token is re-send and checked at each send or recv command to be sure user
     * is authenticated.
     */
    private String authenticationToken = null;

    /** The Http Proxy instance */
    private HttpProxy httpProxy = null;

    /** The Http Parameters instance */
    private HttpProtocolParameters httpProtocolParameters = null;

    /** The calling/owner thread */
    private AwakeProgressManager awakeProgressManager = null;

    /** The http transfer instance */
    private HttpTransfer httpTransfer = null;

    /**
     * Says if we wan to use base64 encoding for parameters passed to call() - 
     * This is a method for legacy applications prior to v1.0.
     */
    public static void setUseBase64EncodingForCall() {
	USE_HTML_ENCODING = false;
    }

    /**
     * Private constructor for clone()
     * 
     * @param url
     *            the URL of the path to the AwakeFileManager Servlet
     * @param username
     *            the username for authentication on the Awake Server (may be
     *            null for call() or downloadUrl())
     * @param authenticationToken
     *            the actual token of the Awake File session to clone
     * @param httpProxy
     *            the http proxy to use
     * @param httpProtocolParameters
     *            the http parameters to use
     */
    private AwakeFileSession(String url, String username,
	    String authenticationToken, HttpProxy httpProxy,
	    HttpProtocolParameters httpProtocolParameters) {
	this.url = url;
	this.username = username;
	this.authenticationToken = authenticationToken;
	this.httpProxy = httpProxy;
	this.httpProtocolParameters = httpProtocolParameters;
    }

    /*
     * Login to the Awake File Server. <br> The Web Behavior of the username
     * operations is as follow: <ul> <li>The method will launch the remote
     * username through HttpsTransferLogin.</li> <li> If successful, remote
     * username will respond with a token that will contain SHA-1(userId +
     * secret value).</li> </ul> The token will be stored in instance memory and
     * resent with each API method and rechecked by the Awake File Server.
     */

    /**
     * Constructor that allows to define a proxy and protocol parameters
     * <p>
     * 
     * @param url
     *            the URL of the path to the AwakeFileManager Servlet
     * @param username
     *            the username for authentication on the Awake Server (may be
     *            null for call() or downloadUrl())
     * @param password
     *            the user password for authentication on the Awake Server (may
     *            be null if password is null)
     * @param httpProxy
     *            the http proxy to use
     * @param httpProtocolParameters
     *            the http parameters to use
     * 
     * @throws MalformedURLException
     *             if the url is malformed
     * @throws UnknownHostException
     *             if Host url (http://www.acme.org) does not exists or no
     *             Internet Connection..
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * @throws InvalidLoginException
     *             the username or password is invalid
     * @throws AwakeRemoteException
     *             an exception has been thrown on the server side. This traps
     *             an Awake product failure and should not happen.
     * @throws AwakeSecurityException
     *             the url is not secured with https (SSL)
     * @throws IOException
     *             For all other IO / Network / System Error
     */
    public AwakeFileSession(String url, String username, char[] password,
	    HttpProxy httpProxy, HttpProtocolParameters httpProtocolParameters)
	    throws IllegalArgumentException, MalformedURLException,
	    UnknownHostException, ConnectException, InvalidLoginException,
	    AwakeRemoteException, AwakeSecurityException, IOException {
	if (url == null) {
	    throw new MalformedURLException("url can not be null!");
	}

	@SuppressWarnings("unused")
	URL asUrl = new URL(url); // Try to raise a MalformedURLException;

	this.username = username;
	this.url = url;

	this.httpProxy = httpProxy;
	this.httpProtocolParameters = httpProtocolParameters;

	// username & password may be null: for call() and downloadUrl() usage
	if (username == null) {
	    return;
	}

	if (password == null) {
	    throw new IllegalArgumentException("password can not be null!");
	}

	String passwordStr = new String(password);

	// Launch the Servlet
	httpTransfer = new HttpTransferOne(url, httpProxy,
		httpProtocolParameters, null);

	// Prepare the request parameters
	List<BasicNameValuePair> requestParams = new Vector<BasicNameValuePair>();
	requestParams.add(new BasicNameValuePair(Parameter.TEST_CRYPTO,
		Parameter.TEST_CRYPTO));
	requestParams.add(new BasicNameValuePair(Parameter.ACTION,
		Action.LOGIN_ACTION));
	requestParams.add(new BasicNameValuePair(Parameter.LOGIN, username));
	requestParams.add(new BasicNameValuePair(Parameter.PASSWORD,
		passwordStr));

	httpTransfer.send(requestParams);

	// If everything is OK, we have in our protocol a response that
	// 1) starts with "OK". 2) Is followed by the Authentication Token
	// else: response starts with "INVALID_LOGIN_OR_PASSWORD".

	String receive = httpTransfer.recv();

	debug("receive: " + receive);

	if (receive.startsWith(ReturnCode.INVALID_LOGIN_OR_PASSWORD)) {
	    throw new InvalidLoginException();
	} else if (receive.startsWith(ReturnCode.OK)) {
	    // OK! We are logged in & and correctly authenticated
	    // Keep in static memory the Authentication Token for next api
	    // commands (First 20 chars)
	    String theToken = receive.substring(ReturnCode.OK.length() + 1);
	    authenticationToken = StringUtils.left(theToken,
		    Parameter.TOKEN_LEFT_SIZE);
	} else {
	    this.username = null;
	    // Should never happen
	    throw new InvalidLoginException();
	}

    }

    /**
     * Constructor that allows to define a proxy.
     * <p>
     * 
     * @param url
     *            the URL of the path to the AwakeFileManager Servlet
     * @param username
     *            the username for authentication on the Awake Server (may be
     *            null for call() or downloadUrl())
     * @param password
     *            the user password for authentication on the Awake Server (may
     *            be null if password is null)
     * @param httpProxy
     *            the http proxy to use
     * 
     * @throws MalformedURLException
     *             if the url is malformed
     * @throws UnknownHostException
     *             if Host url (http://www.acme.org) does not exists or no
     *             Internet Connection..
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * @throws InvalidLoginException
     *             the username or password is invalid
     * @throws AwakeRemoteException
     *             an exception has been thrown on the server side
     * @throws AwakeSecurityException
     *             the url is not secured with https (SSL)
     * @throws IOException
     *             For all other IO / Network / System Error
     */
    public AwakeFileSession(String url, String username, char[] password,
	    HttpProxy httpProxy) throws IllegalArgumentException,
	    MalformedURLException, UnknownHostException, ConnectException,
	    InvalidLoginException, AwakeRemoteException,
	    AwakeSecurityException, IOException {
	this(url, username, password, httpProxy, null);
    }

    /**
     * Constructor.
     * <p>
     * 
     * @param url
     *            the URL of the path to the AwakeFileManager Servlet
     * @param username
     *            the username for authentication on the Awake Server (may be
     *            null for call() or downloadUrl())
     * @param password
     *            the user password for authentication on the Awake Server (may
     *            be null if password is null)
     * 
     * @throws MalformedURLException
     *             if the url is malformed
     * @throws UnknownHostException
     *             if Host url (http://www.acme.org) does not exists or no
     *             Internet Connection..
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * @throws InvalidLoginException
     *             the username or password is invalid
     * @throws AwakeRemoteException
     *             an exception has been thrown on the server side
     * @throws AwakeSecurityException
     *             the url is not secured with https (SSL)
     * @throws IOException
     *             For all other IO / Network / System Error
     */
    public AwakeFileSession(String url, String username, char[] password)
	    throws IllegalArgumentException, MalformedURLException,
	    UnknownHostException, ConnectException, InvalidLoginException,
	    AwakeRemoteException, AwakeSecurityException, IOException {
	this(url, username, password, null, null);
    }

    /**
     * Allows to set an Awake Progress Manager instance.
     * 
     * @param awakeProgressManager
     *            the Awake Progress Manager instance
     */
    public void setAwakeProgressManager(
	    AwakeProgressManager awakeProgressManager) {
	this.awakeProgressManager = awakeProgressManager;
    }

    /**
     * Returns the username of this Awake File session
     * @return the username of this Awake File session
     */
    public String getUsername() {
	return this.username;
    }

    /**
     * Returns the <code>AwakeProgresManager</code> in use.
     * 
     * @return the <code>AwakeProgresManager</code> in use
     */
    public AwakeProgressManager getAwakeProgressManager() {
	return this.awakeProgressManager;
    }

    /**
     * Returns the HttpProtocolParameters instance in use for the Awake File
     * session.
     * 
     * @return the HttpProtocolParameters instance in use for the Awake File
     *         session
     */
    public HttpProtocolParameters getHttpProtocolParameters() {
	return this.httpProtocolParameters;
    }

    /**
     * Returns the URL of the path to the <code>AwakeFileManager</code> Servlet (or
     * <code>AwakeSqlManager</code> Servlet if session has been initiated by an
     * <code>AwakeConnection</code>).
     * 
     * @return the URL of the path to the <code>AwakeFileManager</code> Servlet
     */
    public String getUrl() {
	return url;
    }

    /**
     * Returns the HttpProxy instance in use for this Awake session.
     * 
     * @return the HttpProxy instance in use for this Awake session
     */
    public HttpProxy getHttpProxy() {
	return this.httpProxy;
    }

    /**
     * Returns the http status code of the last executed verb
     * 
     * @return the http status code of the last executed verb
     */
    public int getHttpStatusCode() {
	if (httpTransfer != null) {
	    return httpTransfer.getHttpStatusCode();
	} else {
	    return 0;
	}
    }

    /**
     * Returns the Authentication Token. This method is used by other Awake
     * products (Awake SQL, ...)
     * 
     * @return the Authentication Token
     */
    public String getAuthenticationToken() {
	return this.authenticationToken;
    }

    /**
     * Downloads a file from the remote server.
     * <p>
     * The path of the remote file name is relative depending on the Awake File
     * configuration on the server. The path may be expressed
     * <p>
     * 
     * @param remoteFilePath
     *            the file name on the host
     * @param file
     *            the file to create on the Client side
     * 
     * @throws IllegalArgumentException
     *             if remoteFilePath or file is null
     * @throws InvalidLoginException
     *             if the username is refused by the remote host
     * @throws FileNotFoundException
     *             if the remote file is not found on server
     * 
     * @throws UnknownHostException
     *             if Host url (http://www.acme.org) does not exists or no
     *             Internet Connection.
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * @throws InvalidLoginException
     *             the username or password is invalid
     * @throws InterruptedException
     *             if download is interrupted by user through an
     *             AwakeProgressManager
     * @throws AwakeRemoteException
     *             an exception has been thrown on the server side
     * @throws AwakeSecurityException
     *             the url is not secured with https (SSL)
     * @throws IOException
     *             For all other IO / Network / System Error
     */
    public void downloadFile(String remoteFilePath, File file)
	    throws IllegalArgumentException, InvalidLoginException,
	    FileNotFoundException, UnknownHostException, ConnectException,
	    InterruptedException, AwakeRemoteException, AwakeSecurityException,
	    IOException

    {
	if (remoteFilePath == null) {
	    throw new IllegalArgumentException(
		    "remoteFilePath can not be null!");
	}

	if (file == null) {
	    throw new IllegalArgumentException("file can not be null!");
	}

	if (username == null || authenticationToken == null) {
	    throw new InvalidLoginException();
	}

	// debug("downloadFile Begin");

	httpTransfer = new HttpTransferOne(url, this.httpProxy,
		httpProtocolParameters, awakeProgressManager);

	// Prepare the request parameters
	List<BasicNameValuePair> requestParams = new Vector<BasicNameValuePair>();
	requestParams.add(new BasicNameValuePair(Parameter.ACTION,
		Action.DOWNLOAD_FILE_ACTION));
	requestParams.add(new BasicNameValuePair(Parameter.LOGIN, username));
	requestParams.add(new BasicNameValuePair(Parameter.TOKEN,
		authenticationToken));
	requestParams.add(new BasicNameValuePair(Parameter.FILENAME,
		remoteFilePath));

	httpTransfer.downloadFile(requestParams, file);

	// If everything is OK, we have in our protocol a response that
	// 1) starts with "OK". 2) Is followed by the authenticaiton token
	// else: response starts with "INVALID_LOGIN_OR_PASSWORD".

	// Return the answer
	String receive = httpTransfer.recv();

	if (receive.startsWith(ReturnCode.INVALID_LOGIN_OR_PASSWORD)) {
	    throw new InvalidLoginException();
	}

	if (receive.startsWith(Tag.FileNotFoundException)) {
	    throw new FileNotFoundException("Remote file does not exists: "
		    + remoteFilePath);
	}

    }

    /**
     * Uploads a file on the server.
     * <p>
     * The path of the remote file name is relative depending on the Awake File
     * configuration on the server.
     * <p>
     * 
     * @param file
     *            the file to upload
     * @param remoteFilePath
     *            the file name to use on host.
     * 
     * @throws IllegalArgumentException
     *             if file or remoteFilePath is null
     * @throws InvalidLoginException
     *             if the username is refused by the remote host
     * @throws FileNotFoundException
     *             if the file to upload is not found
     * 
     * @throws UnknownHostException
     *             if Host url (http://www.acme.org) does not exists or no
     *             Internet Connection.
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * 
     * @throws AwakeRemoteException
     *             an exception has been thrown on the server side
     * @throws AwakeSecurityException
     *             the url is not secured with https (SSL)
     * @throws IOException
     *             For all other IO / Network / System Error
     * 
     */
    public void uploadFile(File file, String remoteFilePath)
	    throws IllegalArgumentException, InvalidLoginException,
	    FileNotFoundException, UnknownHostException, ConnectException,
	    InterruptedException, AwakeRemoteException, AwakeSecurityException,
	    IOException

    {
	if (remoteFilePath == null) {
	    throw new IllegalArgumentException(
		    "remoteFilePath can not be null!");
	}

	if (file == null) {
	    throw new IllegalArgumentException("file can not be null!");
	}

	if (!file.exists()) {
	    throw new FileNotFoundException("File does not exists: " + file);
	}

	if (username == null || authenticationToken == null) {
	    throw new InvalidLoginException();
	}

	// Lauch the Servlet
	try {
	    httpTransfer = new HttpTransferOne(url, this.httpProxy,
		    httpProtocolParameters, awakeProgressManager);

	    // Prepare the request parameters
	    List<BasicNameValuePair> requestParams = new Vector<BasicNameValuePair>();
	    requestParams.add(new BasicNameValuePair(Parameter.ACTION,
		    Action.UPLOAD_FILE_ACTION));
	    requestParams.add(new BasicNameValuePair(Parameter.LOGIN, username));
	    requestParams.add(new BasicNameValuePair(Parameter.TOKEN,
		    authenticationToken));
	    requestParams.add(new BasicNameValuePair(Parameter.FILENAME,
		    remoteFilePath));

	    httpTransfer.send(requestParams, file);
	} catch (MyIOInterruptedException e) {
	    throw new InterruptedException();
	}

	// If everything is OK, we have in our protocol a response that
	// 1) starts with "OK". 2) Is followed by the authenticaiton token
	// else: response starts with "INVALID_LOGIN_OR_PASSWORD".

	// Return the answer
	String receive = httpTransfer.recv();

	debug("receive: " + receive);

	if (receive.startsWith(ReturnCode.INVALID_LOGIN_OR_PASSWORD)) {
	    throw new InvalidLoginException();
	}

    }

    /**
     * Returns the length of a a list of files located on the remote host.
     * 
     * @param remoteFilesPath
     *            the list of the files located on the remote host.
     *            <p>
     * @return <ul>
     *         <li>the total length of the files located on the remote host.
     *         </li> <li>or -1 if an error occurred.</li>
     *         </ul>
     * 
     * @throws IllegalArgumentException
     *             remoteFilesPath is null
     * @throws InvalidLoginException
     *             if the username is refused by the remote host
     * 
     * @throws UnknownHostException
     *             if the host url (http://www.acme.org) does not exists or no
     *             Internet Connection.
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * @throws AwakeRemoteException
     *             an exception has been thrown on the server side
     * @throws AwakeSecurityException
     *             the url is not secured with https (SSL)
     * @throws IOException
     *             For all other IO / Network / System Error
     */
    public long getRemoteFilesLength(List<String> remoteFilesPath)
	    throws IllegalArgumentException, InvalidLoginException,
	    UnknownHostException, ConnectException, AwakeRemoteException,
	    AwakeSecurityException, IOException {
	if (remoteFilesPath == null) {
	    throw new IllegalArgumentException(
		    "remotesFilePath can not be null!");
	}

	if (username == null || authenticationToken == null) {
	    throw new InvalidLoginException();
	}

	remoteFilesPath = HtmlConverter.toHtml(remoteFilesPath);

	httpTransfer = new HttpTransferOne(url, httpProxy,
		httpProtocolParameters, awakeProgressManager);

	ListHolder listHolderRemoteFilesPath = new ListHolder();
	listHolderRemoteFilesPath.setList(remoteFilesPath);

	String jsonString = JsonClientBuilder.toJson(listHolderRemoteFilesPath);

	// Prepare the request parameters
	List<BasicNameValuePair> requestParams = new Vector<BasicNameValuePair>();
	requestParams.add(new BasicNameValuePair(Parameter.ACTION,
		Action.GET_FILE_LENGTH_ACTION));
	requestParams.add(new BasicNameValuePair(Parameter.LOGIN, username));
	requestParams.add(new BasicNameValuePair(Parameter.TOKEN,
		authenticationToken));
	requestParams.add(new BasicNameValuePair(Parameter.FILENAME, jsonString));

	httpTransfer.send(requestParams);

	// If everything is OK, we have in our protocol a response that
	// 1) starts with "OK". 2) Is followed by the Authentication Token
	// else: response starts with "INVALID_LOGIN_OR_PASSWORD".

	String response = httpTransfer.recv();

	if (response.startsWith(ReturnCode.INVALID_LOGIN_OR_PASSWORD)) {
	    throw new InvalidLoginException();
	} else {
	    try {
		long fileLength = Long.parseLong(response);
		return fileLength;
	    } catch (NumberFormatException nfe) {
		// Build an Awake Exception with the content of the recv stream
		throw new IOException(Tag.AWAKE_FAIL + nfe.getMessage(), nfe);
	    }
	}
    }

    /**
     * Returns the length of a file located on the remote host.
     * 
     * @param remoteFilePath
     *            the (relative) path of the file located on the remote host.
     * 
     * @return <ul>
     *         <li>the length of the file located on the remote host.</li>
     *         <li>or 0 if the file does not exists on the remote host.</li>
     *         <li>or -1 if an error occurred.</li>
     *         </ul>
     * @throws IllegalArgumentException
     *             if remoteFilesPath is null
     * @throws InvalidLoginException
     *             if the username is refused by the remote host
     * 
     * @throws UnknownHostException
     *             if the Host url (http://www.acme.org) does not exists or no
     *             Internet Connection.
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * @throws AwakeRemoteException
     *             an exception has been thrown on the server side
     * @throws AwakeSecurityException
     *             the url is not secured with https (SSL)
     * @throws IOException
     *             For all other IO / Network / System Error
     */
    public long getRemoteFileLength(String remoteFilePath)
	    throws IllegalArgumentException, InvalidLoginException,
	    UnknownHostException, ConnectException, AwakeRemoteException,
	    AwakeSecurityException, IOException {
	if (remoteFilePath == null) {
	    throw new IllegalArgumentException(
		    "remotesFilePath can not be null!");
	}

	if (username == null || authenticationToken == null) {
	    throw new InvalidLoginException();
	}

	List<String> list = new Vector<String>();
	list.add(remoteFilePath);
	return getRemoteFilesLength(list);
    }

    /**
     * Deletes a remote file or remote directory on the host.
     * (Directory must be empty to be deleted).
     * 
     * @param remoteFilePath
     *            the (relative) path of the file or directory located on the remote host.
     * 
     * @return true if the file has been deleted
     * 
     * @throws IllegalArgumentException
     *             if remoteFilePath is null
     * @throws InvalidLoginException
     *             if the username is refused by the remote host
     * 
     * @throws UnknownHostException
     *             if Host url (http://www.acme.org) does not exists or no
     *             Internet Connection.
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * @throws AwakeRemoteException
     *             an exception has been thrown on the server side
     * @throws AwakeSecurityException
     *             the url is not secured with https (SSL)
     * @throws IOException
     *             For all other IO / Network / System Error
     */
    public boolean deleteRemoteFile(String remoteFilePath)
	    throws IllegalArgumentException, InvalidLoginException,
	    UnknownHostException, ConnectException, AwakeRemoteException,
	    AwakeSecurityException, IOException {
	return actionOnUniqueFile(remoteFilePath, Action.DELETE_FILE_ACTION);
    }

    /**
     * Executes a Java File.mkdirs() on the Host.
     * 
     * @param remoteFilePath
     *            the (relative) path of the directory to create on the remote
     *            host.
     * 
     * @return true if the mkdirs() is successfull
     * 
     * @throws IllegalArgumentException
     *             if remoteFilePath is null
     * @throws InvalidLoginException
     *             the username is refused by the remote host
     * 
     * @throws UnknownHostException
     *             if host url (http://www.acme.org) does not exists or no
     *             Internet Connection.
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * @throws IOException
     *             For all other IO / Network / System Error
     */
    public boolean mkdirsRemote(String remoteFilePath)
	    throws IllegalArgumentException, InvalidLoginException,
	    UnknownHostException, ConnectException, AwakeRemoteException,
	    AwakeSecurityException, IOException {
	return actionOnUniqueFile(remoteFilePath, Action.MKDIRS_ACTION);
    }

    /**
     * 
     * Do a boolean action on a unique file or directory
     * 
     * @param remoteFilePath
     *            the full path to the filename on the host
     * @return true if the file has been deleted
     * 
     * @throws IllegalArgumentException
     *             if remoteFilePath or action is null
     * @throws InvalidLoginException
     *             if the username is refused by the remote host
     * 
     * @throws UnknownHostException
     *             if Host url (http://www.acme.org) does not exists or no
     *             Internet Connection.
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * @throws AwakeRemoteException
     *             an exception has been thrown on the server side
     * @throws AwakeSecurityException
     *             the url is not secured with https (SSL)
     * @throws IOException
     *             For all other IO / Network / System Error
     */
    private boolean actionOnUniqueFile(String remoteFilePath, String action)
	    throws IllegalArgumentException, InvalidLoginException,
	    UnknownHostException, ConnectException, AwakeRemoteException,
	    AwakeSecurityException, IOException {
	if (remoteFilePath == null) {
	    throw new IllegalArgumentException(
		    "remoteFilePath can not be null!");
	}

	if (action == null) {
	    throw new IllegalArgumentException("action can not be null!");
	}

	if (username == null || authenticationToken == null) {
	    throw new InvalidLoginException();
	}

	// Launch the Servlet
	httpTransfer = new HttpTransferOne(url, httpProxy,
		httpProtocolParameters, awakeProgressManager);

	// Prepare the request parameters
	List<BasicNameValuePair> requestParams = new Vector<BasicNameValuePair>();
	requestParams.add(new BasicNameValuePair(Parameter.ACTION, action));
	requestParams.add(new BasicNameValuePair(Parameter.LOGIN, username));
	requestParams.add(new BasicNameValuePair(Parameter.TOKEN,
		authenticationToken));
	requestParams.add(new BasicNameValuePair(Parameter.FILENAME,
		remoteFilePath));

	httpTransfer.send(requestParams);

	// If everything is OK, we have in our protocol a response that
	// 1) starts with "OK". 2) Is followed by the authenticaiton token
	// else: response starts with "INVALID_LOGIN_OR_PASSWORD".
	String response = httpTransfer.recv();

	if (response.startsWith(ReturnCode.INVALID_LOGIN_OR_PASSWORD)) {
	    throw new InvalidLoginException();
	}

	try {
	    boolean actionDone = Boolean.parseBoolean(response);
	    return actionDone;
	} catch (NumberFormatException nfe) {
	    // Build an Awake Exception with the content of the recv stream
	    throw new IOException(Tag.AWAKE_FAIL + nfe.getMessage(), nfe);
	}
    }

    /**
     * Lists the files contained in a remote directory.
     * 
     * @param remoteFilePath
     *            the full path to the filename on the host
     * @return true if the file has been deleted
     * 
     * @throws IllegalArgumentException
     *             if remoteFilePath is null
     * @throws InvalidLoginException
     *             if the username is refused by the remote host
     * 
     * @throws UnknownHostException
     *             if Host url (http://www.acme.org) does not exists or no
     *             Internet Connection.
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * @throws AwakeRemoteException
     *             an exception has been thrown on the server side
     * @throws AwakeSecurityException
     *             the url is not secured with https (SSL)
     * @throws IOException
     *             For all other IO / Network / System Error
     */
    public List<String> listRemoteFiles(String remoteFilePath)
	    throws IllegalArgumentException, InvalidLoginException,
	    UnknownHostException, ConnectException, AwakeRemoteException,
	    AwakeSecurityException, IOException {
	return listFilesOrDirectory(remoteFilePath,
		Action.LIST_FILES_IN_DIR_ACTION);
    }

    /**
     * Lists the sub-directories contained in a remote directory
     * 
     * @param remoteFilePath
     *            the full path to the filename on the host
     * @return true if the file has been deleted
     * 
     * @throws IllegalArgumentException
     *             if remoteFilePath is null
     * @throws InvalidLoginException
     *             if the username is refused by the remote host
     * 
     * @throws UnknownHostException
     *             if Host url (http://www.acme.org) does not exists or no
     *             Internet Connection.
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * @throws AwakeRemoteException
     *             an exception has been thrown on the server side
     * @throws AwakeSecurityException
     *             the url is not secured with https (SSL)
     * @throws IOException
     *             For all other IO / Network / System Error
     */
    public List<String> listRemoteDirectories(String remoteFilePath)
	    throws IllegalArgumentException, InvalidLoginException,
	    UnknownHostException, ConnectException, AwakeRemoteException,
	    AwakeSecurityException, IOException {
	return listFilesOrDirectory(remoteFilePath,
		Action.LIST_DIRS_IN_DIR_ACTION);
    }

    /**
     * 
     * Returns the list of files or directory in the remote directory.
     * 
     * @param remoteFilePath
     *            the full path to the filename on the host
     * @return true if the file has been deleted
     * 
     * @throws IllegalArgumentException
     *             if remoteFilePath is null
     * @throws InvalidLoginException
     *             if the username is refused by the remote host
     * 
     * @throws UnknownHostException
     *             if Host url (http://www.acme.org) does not exists or no
     *             Internet Connection.
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * @throws AwakeRemoteException
     *             an exception has been thrown on the server side
     * @throws AwakeSecurityException
     *             the url is not secured with https (SSL)
     * @throws IOException
     *             For all other IO / Network / System Error
     */
    private List<String> listFilesOrDirectory(String remoteFilePath,
	    String action) throws IllegalArgumentException,
	    InvalidLoginException, UnknownHostException, ConnectException,
	    AwakeRemoteException, AwakeSecurityException, IOException

    {
	if (remoteFilePath == null) {
	    throw new IllegalArgumentException(
		    "remoteFilePath can not be null!");
	}

	if (action == null) {
	    throw new IllegalArgumentException("action can not be null!");
	}

	if (username == null || authenticationToken == null) {
	    throw new InvalidLoginException();
	}

	// Launch the Servlet
	httpTransfer = new HttpTransferOne(url, httpProxy,
		httpProtocolParameters, awakeProgressManager);

	// Prepare the request parameters
	List<BasicNameValuePair> requestParams = new Vector<BasicNameValuePair>();
	requestParams.add(new BasicNameValuePair(Parameter.ACTION, action));
	requestParams.add(new BasicNameValuePair(Parameter.LOGIN, username));
	requestParams.add(new BasicNameValuePair(Parameter.TOKEN,
		authenticationToken));
	requestParams.add(new BasicNameValuePair(Parameter.FILENAME,
		remoteFilePath));

	httpTransfer.setReceiveInFile(true); // To say we get the result into a
					     // file
	httpTransfer.send(requestParams);

	// If everything is OK, we have in our protocol a response that
	// 1) starts with "OK". 2) Is followed by the authenticaiton token
	// else: response starts with "INVALID_LOGIN_OR_PASSWORD".

	File receiveFile = httpTransfer.getReceiveFile();
	String receive = httpTransfer.recv();

	// Content is OK
	if (receive.startsWith(ReturnCode.INVALID_LOGIN_OR_PASSWORD)) {
	    if (!DEBUG && ! KeepTempFilePolicyParms.KEEP_TEMP_FILE) {
		receiveFile.delete();
	    }
	    throw new InvalidLoginException();
	}

	try {
	    if (receive.equals("null")) {
		return new Vector<String>();
	    } else {

		List<String> list = getFilesListFromFile(receiveFile);
		return list;
	    }
	} catch (Exception e) {
	    throw new IOException(e.getMessage(), e);
	}
	finally
	{
	    if (!DEBUG && ! KeepTempFilePolicyParms.KEEP_TEMP_FILE) {
		receiveFile.delete();
	    }
	}

    }

    /**
     * Transforms the content of the file in Base64 lines into a list.
     * 
     * @param file
     *            the file containing the file names in Base64 encoding
     * 
     * @return the content of the file in Base64 lines into a list
     * @throws IOException
     *             if any IO / Network / System Error occurs
     */
    private List<String> getFilesListFromFile(File file) throws IOException {
	
	List<String> listBase = FileUtils.readLines(file);

	List<String> files = new Vector<String>();

	for (String theFileStr : listBase) {

	    theFileStr = HtmlConverter.fromHtml(theFileStr);

	    files.add(theFileStr);
	}

	return files;
    }

    /**
     * Logs off from the remote host.&nbsp; This will purge the authentication
     * values necessary for method calls.
     * <p>
     * <b>Method should be called at the closure of the Client application</b>.
     */
    public void logoff() {
	username = null;
	authenticationToken = null;
    }

    
    /**
     * Calls a remote Java {@code class.method} and (eventually) pass some parameters to
     * it. 
     * This method transforms the values in Base64. 
     * It's a legacy method not to be used anymore: use {@code call} instead.
     * 
     * @param methodName
     *            the full method name to call in the format
     *            <code>org.acme.config.package.MyClass.myMethod</code>
     * @param params
     *            the array of parameters passed to the method
     * @return the result of the Java call as string
     * 
     * @throws IllegalArgumentException
     *             if methodName is null
     * @throws InvalidLoginException
     *             if the username is refused by the remote host
     * 
     * @throws UnknownHostException
     *             if Host url (http://www.acme.org) does not exists or no
     *             Internet Connection.
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * @throws AwakeRemoteException
     *             an exception has been thrown on the server side
     * @throws AwakeSecurityException
     *             the method is not allowed to be called by server side
     * @throws IOException
     *             For all other IO / Network / System Error
     *             
     */
    
    private String callBase64Encoded(String methodName, Object... params)
	    throws IllegalArgumentException, InvalidLoginException,
	    UnknownHostException, ConnectException, AwakeRemoteException,
	    AwakeSecurityException, IOException {

	// Class and method name can not be null
	if (methodName == null) {
	    throw new IllegalArgumentException("methodName can not be null!");
	}

	// username & Authentication Token may be null
	// because some methods can be called freely

	if (username == null) {
	    username = "null";
	}

	if (authenticationToken == null) {
	    authenticationToken = "null";
	}

	// Build the params types
	List<String> paramsTypes = new Vector<String>();

	// Build the params values
	List<String> paramsValues = new Vector<String>();

	debug("");

	for (int i = 0; i < params.length; i++) {
	    if (params[i] == null) {
		throw new IllegalArgumentException(
			Tag.AWAKE
				+ "null values are not supported. Please provide a value for all parameters.");
	    } else {
		String classType = params[i].getClass().getName();

		// NO! can alter class name if value is obsfucated
		// classType = StringUtils.substringAfterLast(classType, ".");
		paramsTypes.add(classType);

		String value = params[i].toString();

		debug("");
		debug("classType: " + classType);
		debug("value    : " + value);

		paramsValues.add(value);
	    }

	}

	ListHolder listHolderTypes = new ListHolder();
	listHolderTypes.setList(paramsTypes);
	String jsonParamTypes = JsonClientBuilder.toJson(listHolderTypes);

	ListHolder listHolderValues = new ListHolder();
	listHolderValues.setList(paramsValues);
	String jsonParamValues = JsonClientBuilder.toJson(listHolderValues);

	debug("methodName     : " + methodName);
	debug("jsonParamTypes : " + jsonParamTypes);
	debug("jsonParamValues: " + jsonParamValues);

	httpTransfer = new HttpTransferOne(url, httpProxy,
		httpProtocolParameters, awakeProgressManager);
	// Prepare the request parameters
	List<BasicNameValuePair> requestParams = new Vector<BasicNameValuePair>();
	requestParams.add(new BasicNameValuePair(Parameter.ACTION,
		Action.CALL_ACTION));
	// requestParams.add(new BasicNameValuePair(Parameter.LOGIN,
	// StringUtil.toBase64(username)));
	requestParams.add(new BasicNameValuePair(Parameter.LOGIN, username));
	requestParams.add(new BasicNameValuePair(Parameter.TOKEN,
		authenticationToken));
	requestParams.add(new BasicNameValuePair(Parameter.METHOD_NAME,
		methodName));
	requestParams.add(new BasicNameValuePair(Parameter.PARAMS_TYPES,
		StringUtil.toBase64(jsonParamTypes)));
	requestParams.add(new BasicNameValuePair(Parameter.PARAMS_VALUES,
		StringUtil.toBase64(jsonParamValues)));

	httpTransfer.send(requestParams);

	// Return the answer
	String response = httpTransfer.recv();

	debug("response: " + response);

	// Content is OK
	if (response.startsWith(ReturnCode.INVALID_LOGIN_OR_PASSWORD)) {
	    throw new InvalidLoginException();
	}

	// The response is in Base 64
	try {
	    if (!response.isEmpty()) {
		response = StringUtil.fromBase64(response);
	    }
	} catch (Exception e) {
	    debug(response);
	    // May happen till new Awake File is not deployed on Server
	    throw new IOException(Tag.AWAKE_FAIL
		    + "Response must be and is not base64!", e);
	}

	return response;

    }

    /**
     * Calls a remote Java class.method and (eventually) pass some parameters to
     * it.
     * 
     * @param methodName
     *            the full method name to call in the format
     *            <code>org.acme.config.package.MyClass.myMethod</code>
     * @param params
     *            the array of parameters passed to the method
     * @return the result of the Java call as string
     * 
     * @throws IllegalArgumentException
     *             if methodName is null
     * @throws InvalidLoginException
     *             if the username is refused by the remote host
     * 
     * @throws UnknownHostException
     *             if Host url (http://www.acme.org) does not exists or no
     *             Internet Connection.
     * @throws ConnectException
     *             if the Host is correct but the AwakeFileManager Servlet is
     *             not reachable (http://www.acme.org/AwakeFileManager) and
     *             access failed with a status != OK (200). (If the host is
     *             incorrect, or is impossible to connect to - Tomcat down - the
     *             ConnectException will be the sub exception
     *             HttpHostConnectException.)
     * @throws AwakeRemoteException
     *             an exception has been thrown on the server side
     * @throws AwakeSecurityException
     *             the method is not allowed to be called by server side
     * @throws IOException
     *             For all other IO / Network / System Error
     * 
     */

    public String call(String methodName, Object... params)
	    throws IllegalArgumentException, InvalidLoginException,
	    UnknownHostException, ConnectException, AwakeRemoteException,
	    AwakeSecurityException, IOException {

	// For legacy methods
	if (! USE_HTML_ENCODING) {
	    return callBase64Encoded(methodName, params);
	}
	
	// Class and method name can not be null
	if (methodName == null) {
	    throw new IllegalArgumentException("methodName can not be null!");
	}

	// username & Authentication Token may be null
	// because some methods can be called freely

	if (username == null) {
	    username = "null";
	}

	if (authenticationToken == null) {
	    authenticationToken = "null";
	}

	// Build the params types
	List<String> paramsTypes = new Vector<String>();

	// Build the params values
	List<String> paramsValues = new Vector<String>();

	debug("");

	for (int i = 0; i < params.length; i++) {
	    if (params[i] == null) {
		throw new IllegalArgumentException(
			Tag.AWAKE
				+ "null values are not supported. Please provide a value for all parameters.");
	    } else {
		String classType = params[i].getClass().getName();

		// NO! can alter class name if value is obsfucated
		// classType = StringUtils.substringAfterLast(classType, ".");
		paramsTypes.add(classType);

		String value = params[i].toString();

		debug("");
		debug("classType: " + classType);
		debug("value    : " + value);

		paramsValues.add(value);
	    }
	}

	ListHolder listHolderTypes = new ListHolder();
	listHolderTypes.setList(paramsTypes);
	String jsonParamTypes = JsonClientBuilder.toJson(listHolderTypes);

	ListHolder listHolderValues = new ListHolder();
	listHolderValues.setList(paramsValues);
	String jsonParamValues = JsonClientBuilder.toJson(listHolderValues);

	debug("methodName     : " + methodName);
	debug("jsonParamTypes : " + jsonParamTypes);
	debug("jsonParamValues: " + jsonParamValues);

	httpTransfer = new HttpTransferOne(url, httpProxy,
		httpProtocolParameters, awakeProgressManager);
	// Prepare the request parameters
	List<BasicNameValuePair> requestParams = new Vector<BasicNameValuePair>();
	requestParams.add(new BasicNameValuePair(Parameter.ACTION,
		Action.CALL_ACTION_HTML_ENCODED));
	requestParams.add(new BasicNameValuePair(Parameter.LOGIN, username));
	requestParams.add(new BasicNameValuePair(Parameter.TOKEN,
		authenticationToken));
	requestParams.add(new BasicNameValuePair(Parameter.METHOD_NAME,
		methodName));
	requestParams.add(new BasicNameValuePair(Parameter.PARAMS_TYPES, jsonParamTypes));
	requestParams.add(new BasicNameValuePair(Parameter.PARAMS_VALUES, jsonParamValues));

	httpTransfer.send(requestParams);

	// Return the answer
	String response = httpTransfer.recv();

	debug("response: " + response);

	// Content is OK
	if (response.startsWith(ReturnCode.INVALID_LOGIN_OR_PASSWORD)) {
	    throw new InvalidLoginException();
	}

	// The response is in Html encode:
	if (!response.isEmpty()) {
	    response = HtmlConverter.fromHtml(response);
	}

	return response;

    }    
    
    /**
     * Allows to get a copy of the current <code>AwakeFileSession</code>: use it to
     * do some simultaneous operations in a different thread (in order to avoid conflicts).
     */
    @Override
    public AwakeFileSession clone() {
	AwakeFileSession awakeFileSession = new AwakeFileSession(this.url,
		this.username, this.authenticationToken, this.httpProxy,
		this.httpProtocolParameters);
	return awakeFileSession;
    }

    /**
     * Returns the Awake File Version.
     * 
     * @return the Awake File Version
     */
    public String getVersion() {
	return AwakeFileVersion.getVersion();
    }

    /**
     * debug tool
     */
    private void debug(String s) {
	if (DEBUG) {
	    AwakeLogger.log(s);
	}
    }

}

// End
