/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2011, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//ComputerInfo.java
//Copyright (c) Kawan Softwares S.A.S, 2011
//
//Last Updates: 
// 3 juin 2011 20:09:43 Nicolas de Pomereu
// 08/06/11 10:45 NDP : ComputerInfo: clean trap of exceptions on getMacAddress();
// 18/07/11 19:05 NDP : ComputerInfo: moved into org.awakefw.file.api.util

package org.awakefw.file.api.util;

import java.net.InetAddress;
import java.net.NetworkInterface;

/**
 * @author Nicolas de Pomereu
 * 
 *         Get some computer infos: host name & MAC address
 */
public class ComputerInfo {
    /** The hostname */
    private static String hostName = null;

    /** The MAC address */
    private static String macAddress = null;

    /**
     * Protected
     */
    protected ComputerInfo() {
    }

    /**
     * Return the computer name (hostname)
     * 
     * @return the name or <b>null</b> if the name cannot be found
     */

    public static String getHostname() {
	try {
	    if (hostName == null) {
		final InetAddress addr = InetAddress.getLocalHost();
		hostName = new String(addr.getHostName());

		if (hostName == null) {
		    hostName = "unknown_hostname";
		}
	    }
	} catch (final Exception e) {
	    hostName = "unknown_hostname";
	    e.printStackTrace(System.out);
	}

	return hostName;
    }

    /**
     * Return the computer MAC address
     * 
     * @return the name or <b>null</b> if the MAC address cannot be found
     */

    public static String getMacAddress() {
	try {
	    if (macAddress == null) {
		InetAddress ip = InetAddress.getLocalHost();

		NetworkInterface network = NetworkInterface
			.getByInetAddress(ip);

		if (network == null) {
		    macAddress = "unknown_mac_address";
		    return macAddress;
		}

		byte[] mac = network.getHardwareAddress();

		if (mac == null) {
		    macAddress = "unknown_mac_address";
		    return macAddress;
		}

		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < mac.length; i++) {
		    sb.append(String.format("%02X%s", mac[i],
			    (i < mac.length - 1) ? "-" : ""));
		}

		macAddress = sb.toString();
	    }
	} catch (Exception e) {
	    macAddress = "unknown_mac_address";
	    e.printStackTrace(System.out);
	}

	return macAddress;
    }

}
