/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2011, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

//RequestInfoStore.java
//Copyright (c) Kawan Softwares S.A.S, 2011
//
//Last Updates: 
// 26 avr. 2011 20:44:34 Nicolas de Pomereu

package org.awakefw.file.api.util;

import javax.servlet.http.HttpServletRequest;

/**
 * @author Nicolas de Pomereu
 * 
 *         Stores the host info. Can be used to query the host in use.
 */

public class RequestInfoStore {

    private static String scheme = null;
    private static String serverName = null;
    private static int port = 0;

    /**
     * Constructor to be used for query
     */
    public RequestInfoStore() {

    }

    /**
     * Sets the HttpServletRequest in memory
     * 
     * @param theRequest
     *            the HttpServletRequest
     */
    public static void init(HttpServletRequest request) {
	if (request == null) {
	    throw new IllegalArgumentException("request can not be null!");
	}

	scheme = request.getScheme();
	serverName = request.getServerName();
	port = request.getServerPort();
    }

    /**
     * Return the host url in format https://www.safester.net
     * 
     * @return host url in format https://www.safester.net
     */
    public String getHostUrl() {
	if (scheme == null || serverName == null) {
	    return null;
	}

	String hostUrl = scheme + "://" + serverName;
	return hostUrl;
    }

    /**
     * @return
     * @see javax.servlet.ServletRequest#getScheme()
     */
    public String getScheme() {
	return scheme;
    }

    /**
     * @return
     * @see javax.servlet.ServletRequest#getServerName()
     */
    public String getServerName() {
	return serverName;
    }

    /**
     * @return
     * @see javax.servlet.ServletRequest#getServerPort()
     */
    public int getServerPort() {
	return port;
    }

}
