/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2011, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

package org.awakefw.file.servlet;

// ServerCallerRecv : SimplifyPost - Execute any remote method
// 
// 17/12/05 12:50 NDP  - Creation
// 17/12/05 19:40 NDP  - Trim all parameters values
// 18/12/05 21:35 NDP  - add crSize & test all
// 09/06/07 12:37 ABE - Clean code to avoid warnings
// 
// 24/07/08 20:15 NDP - Clean code & new token mechanism
// 19/08/08 19:40 NDP - login &  Authentication Token may be null
// 02/03/10 15:25 NDP - AwakeFileServer: New name 
// 17/10/10 19:50 NDP : AwakeFileServer: Wrap the HttpServletRequest with HttpServletRequestEncrypted 
//                      for parameters decryption encrypted on the PC
// 15/03/11 17:20 NDP - AwakeFileServer: no more awakeJdbcDriverConfiguratorClassName
// 27/04/11 19:05 NDP - AwakeFileServer: put as much init code in init() instead of doPost()
// 20/06/11 12:45 NDP : AwakeFileManager: new name for AwakeFileServer
// 23/09/11 17:30 NDP : AwakeFileManager: better fault tolerance on first class name capitalization
// 10/10/11 12:55 NDP : AwakeFileManager: add get method to display configuration info
// 10/10/11 15:25 NDP : AwakeFileManager: add debug info configurator
// 14/11/11 17:45 NDP : AwakeFileManager: call AwakeLogger.createLogger(directory);
// 21/11/11 16:45 NDP : AwakeFileManager: cleaner display of init parameters

import java.io.IOException;
import java.io.PrintWriter;
import java.util.logging.Logger;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.awakefw.commons.api.server.AwakeCommonsConfigurator;
import org.awakefw.commons.api.server.DefaultAwakeCommonsConfigurator;
import org.awakefw.commons.server.util.AwakeLogger;
import org.awakefw.file.api.server.AwakeFileConfigurator;
import org.awakefw.file.api.server.DefaultAwakeFileConfigurator;
import org.awakefw.file.api.server.fileaction.AwakeFileActionManager;
import org.awakefw.file.api.server.fileaction.DefaultAwakeFileActionManager;
import org.awakefw.file.api.util.RequestInfoStore;
import org.awakefw.file.util.Tag;
import org.awakefw.file.util.convert.HttpServletRequestConvertor;

/**
 * The main Awake File Manager Servlet <br>
 * 
 * @author Nicolas de Pomereu
 */

@SuppressWarnings("serial")
public class AwakeFileManager extends HttpServlet {

    public static final String AWAKE_FILE_ACTION_MANAGER_CLASS_NAME = "awakeFileActionManagerClassName";
    public static final String AWAKE_FILE_CONFIGURATOR_CLASS_NAME = "awakeFileConfiguratorClassName";
    public static final String AWAKE_COMMONS_CONFIGURATOR_CLASS_NAME = "awakeCommonsConfiguratorClassName";

    private static AwakeCommonsConfigurator awakeCommonsConfigurator = null;
    private static AwakeFileConfigurator awakeFileConfigurator = null;
    private static AwakeFileActionManager awakeFileActionManager = null;

    /**
     * Init.
     */

    public void init(ServletConfig config) throws ServletException {
	super.init(config);
	String awakeCommonsConfiguratorClassName = config
		.getInitParameter(AWAKE_COMMONS_CONFIGURATOR_CLASS_NAME);
	String awakeFileConfiguratorClassName = config
		.getInitParameter(AWAKE_FILE_CONFIGURATOR_CLASS_NAME);
	String awakeFileActionManagerClassName = config
		.getInitParameter(AWAKE_FILE_ACTION_MANAGER_CLASS_NAME);

	// Check spelling with first letter capitalized
	if (awakeCommonsConfiguratorClassName == null
		|| awakeCommonsConfiguratorClassName.isEmpty()) {
	    String capitalized = StringUtils
		    .capitalize(AWAKE_COMMONS_CONFIGURATOR_CLASS_NAME);
	    awakeCommonsConfiguratorClassName = config
		    .getInitParameter(capitalized);
	}

	if (awakeFileConfiguratorClassName == null
		|| awakeFileConfiguratorClassName.isEmpty()) {
	    String capitalized = StringUtils
		    .capitalize(AWAKE_FILE_CONFIGURATOR_CLASS_NAME);
	    awakeFileConfiguratorClassName = config
		    .getInitParameter(capitalized);
	}

	if (awakeFileActionManagerClassName == null
		|| awakeFileActionManagerClassName.isEmpty()) {
	    String capitalized = StringUtils
		    .capitalize(AWAKE_FILE_ACTION_MANAGER_CLASS_NAME);
	    awakeFileActionManagerClassName = config
		    .getInitParameter(capitalized);
	}

	System.out.println(Tag.AWAKE_START
		+ org.awakefw.file.version.AwakeFileVersion.getVersion());
	System.out.println(Tag.AWAKE_START + this.getClass().getName() + " Servlet:");		
	System.out.println(Tag.AWAKE_START
		+ "- Init Parameter awakeCommonsConfiguratorClassName = "
		+ awakeCommonsConfiguratorClassName);
	System.out.println(Tag.AWAKE_START
		+ "- Init Parameter awakeFileConfiguratorClassName    = "
		+ awakeFileConfiguratorClassName);
	System.out.println(Tag.AWAKE_START
		+ "- Init Parameter awakeFileActionManagerClassName   = "
		+ awakeFileActionManagerClassName);

	// System.out.println(Tag.AWAKE_START + "AwakeFileManager start...");

	// Variable use to store the current name when loading, used to detail
	// the exception in the catch clauses
	String classNameToLoad = null;

	// Call the specific Awake File Configurator class to use
	try {

	    classNameToLoad = awakeCommonsConfiguratorClassName;
	    if (awakeCommonsConfiguratorClassName != null
		    && !awakeCommonsConfiguratorClassName.isEmpty()) {
		Class<?> c = Class.forName(awakeCommonsConfiguratorClassName);
		awakeCommonsConfigurator = (AwakeCommonsConfigurator) c
			.newInstance();
	    } else {
		awakeCommonsConfigurator = new DefaultAwakeCommonsConfigurator();
	    }
	    
	    // Immediately create the AwakeLogger
	    Logger logger = null;
	    try {
		logger = awakeCommonsConfigurator.getLogger();
		AwakeLogger.createLogger(logger);
	    } catch (Exception e) {
		throw new ServletException(Tag.AWAKE_CONFIG_FAIL 
			+ "Impossible to create the Logger: " + logger , e);
	    }
	    
	    classNameToLoad = awakeFileConfiguratorClassName;
	    if (awakeFileConfiguratorClassName != null
		    && !awakeFileConfiguratorClassName.isEmpty()) {
		Class<?> c = Class.forName(awakeFileConfiguratorClassName);
		awakeFileConfigurator = (AwakeFileConfigurator) c.newInstance();
	    } else {
		awakeFileConfigurator = new DefaultAwakeFileConfigurator();
	    }

	    classNameToLoad = awakeFileActionManagerClassName;
	    if (awakeFileActionManagerClassName != null
		    && !awakeFileActionManagerClassName.isEmpty()) {
		Class<?> c = Class.forName(awakeFileActionManagerClassName);
		awakeFileActionManager = (AwakeFileActionManager) c
			.newInstance();
	    } else {
		awakeFileActionManager = new DefaultAwakeFileActionManager();
	    }

	} catch (ClassNotFoundException e) {
	    String errrorMesage = Tag.AWAKE_CONFIG_FAIL
		    + "Impossible to find Awake Configurator class: "
		    + classNameToLoad;
	    System.out.println(errrorMesage);
	    throw new ServletException(errrorMesage, e);
	} catch (InstantiationException e) {
	    String errrorMesage = Tag.AWAKE_CONFIG_FAIL
		    + "Impossible to load Awake Configurator class: "
		    + classNameToLoad;
	    System.out.println(errrorMesage);
	    throw new ServletException(errrorMesage, e);
	} catch (IllegalAccessException e) {
	    String errrorMesage = Tag.AWAKE_CONFIG_FAIL
		    + "Impossible to load Awake File Configurator class: "
		    + classNameToLoad;
	    System.out.println(errrorMesage);
	    throw new ServletException(errrorMesage, e);
	}

    }

    /**
     * Post request.
     */
    @Override
    public void doPost(HttpServletRequest request, HttpServletResponse response)
	    throws IOException {
	// Store the host info in RequestInfoStore
	RequestInfoStore.init(request);

	// Wrap the HttpServletRequest with HttpServletRequestEncrypted for
	// parameters decryption
	HttpServletRequestConvertor requestEncrypted = new HttpServletRequestConvertor(
		request, awakeCommonsConfigurator);

	// System.out.println(Tag.AWAKE_START + "AwakeFileManager started!");
	ServerAwakeFileDispatch dispatch = new ServerAwakeFileDispatch();
	dispatch.executeRequest(requestEncrypted, response,
		awakeCommonsConfigurator, awakeFileConfigurator,
		awakeFileActionManager);
    }

    /**
     * Get request.
     */
    @Override
    public void doGet(HttpServletRequest request, HttpServletResponse response)
	    throws IOException {
	response.setContentType("text/html");
	PrintWriter out = response.getWriter();

	out.println(this.getClass().getName() + " Servlet Configuration");
	out.println("<br><br>");
	out.println("<table cellpadding=\"3\" border=\"1\">");

	out.println("<tr>");
	out.println("<td align=\"center\"> Parameter </td>");
	out.println("<td align=\"center\"> Value </td>");
	out.println("</tr>");

	out.println("<tr>");
	out.println("<td> " + AWAKE_COMMONS_CONFIGURATOR_CLASS_NAME + "</td>");
	out.println("<td> " + awakeCommonsConfigurator.getClass().getName()
		+ "</td>");
	out.println("</tr>");

	out.println("<tr>");
	out.println("<td> " + AWAKE_FILE_CONFIGURATOR_CLASS_NAME + "</td>");
	out.println("<td> " + awakeFileConfigurator.getClass().getName()
		+ "</td>");
	out.println("</tr>");

	out.println("<tr>");
	out.println("<td> " + AWAKE_FILE_ACTION_MANAGER_CLASS_NAME + "</td>");
	out.println("<td> " + awakeFileActionManager.getClass().getName()
		+ "</td>");
	out.println("</tr>");

	out.println("</table>");

    }

}
