/*
 * Awake File: Easy file upload & download through HTTP with Java
 * Awake SQL: Remote JDBC access through HTTP.                                    
 * Copyright (C) 2011, Kawan Softwares S.A.S.
 * (http://www.awakeframework.org). All rights reserved.                                
 *                                                                               
 * Awake File/SQL is free software; you can redistribute it and/or                 
 * modify it under the terms of the GNU Lesser General Public                    
 * License as published by the Free Software Foundation; either                  
 * version 2.1 of the License, or (at your option) any later version.            
 *                                                                               
 * Awake File/SQL is distributed in the hope that it will be useful,               
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             
 * Lesser General Public License for more details.                               
 *                                                                               
 * You should have received a copy of the GNU Lesser General Public              
 * License along with this library; if not, write to the Free Software           
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301  USA
 *
 * Any modifications to this file must keep this entire header
 * intact.
 */

// StringMgr.java	 : Kawan Softwares S.A.S Security Package.
// Class Description : Static methods for converting String
//
// Copyright (c) Nicolas de Pomereu 1999
//
// Last Updates: 
// 09/08/99 19:45 NDP - Add package & generalize
// 15/09/00 19:15 NDP - Add UTF converters
// 10/09/02 17:35 NDP - Add ReplaceFirst
// 21/10/02 16:05 NDP - handle new special characters
// 12/11/02 18:20 NDP - add replaceAllIgnorecase
// 16/11/02 18:35 NDP - add countSubstring
// 19/11/02 11:15 NDP - add replaceFirstIgnorecase
// 08/07/03 20:25 NDP - optimize ReplaceXxx() methods with StringBuffer()
// 12/07/03 19:30 NDP - Use of Regular expression is forbidden in split 
// 12/07/03 19:45 NDP - remove all regex special chars
// 14/07/03 10:00 NDP - comments 
// 21/07/03 12:40 NDP - comments 
// 10/10/08 11:45 NDP - StringMgr.ReplaceAll ==> test if input is not null
// 12/03/09 16:35 NDP - Add uppercase accents in RemoveAccents()

package org.awakefw.file.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.ArrayList;

public class StringMgr {

    /**
     * Convert a byte array to a byte array in UTF format <br>
     * Uses an underlying byte array output stream
     * 
     * @param s
     *            a String to convert in UTF formated bytes
     * @return a byte array in UTF format
     */

    public static byte[] toUTFBytes(String s) throws IOException {
	if (s == null) {
	    return null;
	}

	ByteArrayOutputStream bos = new ByteArrayOutputStream();
	DataOutputStream dos = new DataOutputStream(bos);
	dos.writeUTF(s);
	dos.close();

	byte[] bBos = bos.toByteArray();
	return bBos;
    }

    /**
     * Convert a byte array in UTF format to a String <br>
     * Uses an underlying byte array input stream
     * 
     * @param bBos
     *            a byte array in UTF format s
     * @return a readable String
     */

    public static String FromUTFBytes(byte[] bBos) throws IOException {
	String sUTF = new String();

	ByteArrayInputStream bis = new ByteArrayInputStream(bBos);
	DataInputStream dis = new DataInputStream(bis);
	sUTF = dis.readUTF();
	dis.close();

	return sUTF;
    }

    /**
     * 
     * split a String using a fast algorithm
     * <p>
     * 
     * @param sString
     *            The String to split.
     * @param sExpression
     *            the expression on wich to split
     *            <p>
     * @return A String with Substring replacement.
     * 
     */

    public static String[] split(String sIn, String sExpression) {
	if (sIn == null) {
	    return null;
	}

	// if (false) return sIn.split(sExpression);

	// Special case : sIn is ""
	if (sIn.equals("")) {
	    String[] sArray = new String[1];
	    sArray[0] = "";
	    return sArray;
	}

	// 12/07/03 19:45 NDP - remove all regex special chars
	if (sExpression.indexOf("\\") != -1) {
	    sExpression = replaceAll(sExpression, "\\", "");
	}

	if (sIn.equals(sExpression)) {
	    return new String[0];
	}

	ArrayList<String> arrayList = new ArrayList<String>();
	int nLen = sExpression.length();
	int nIndex = 0;

	String sBegin = null;

	while (true) {
	    nIndex = sIn.indexOf(sExpression);

	    if (nIndex == -1) {
		if (sIn.length() != 0) {
		    arrayList.add(sIn);
		}
		break;
	    }

	    if (sIn.length() == 0) {
		break;
	    }

	    if (nLen == 0) {
		sBegin = sIn.substring(0, nIndex + 1);
		sIn = sIn.substring(nIndex + nLen + 1);
	    } else {
		sBegin = sIn.substring(0, nIndex);
		sIn = sIn.substring(nIndex + nLen);
	    }

	    arrayList.add(sBegin);

	}

	String[] sArray = new String[arrayList.size()];

	for (int i = 0; i < arrayList.size(); i++) {
	    sArray[i] = (String) arrayList.get(i);
	}

	return sArray;

    }

    /**
     * 
     * Replace first occurence of a Substring by a new Substring in a String
     * <p>
     * 
     * @param sString
     *            The String to modify.
     * @param sOldChars
     *            The Substring to replace.
     * @param sNewChars
     *            The new Substring to replace with.
     *            <p>
     * @return A String with Substring replacement.
     * 
     */

    public static String replaceFirst(String sString, String sOldSub,
	    String sNewSub) {

	if (sString == null) {
	    return null;
	}

	StringBuffer sbNew = new StringBuffer();
	String sOld = sString;

	int nOldLength = sOldSub.length();
	int nBindex = 0;

	if (sOld.indexOf(sOldSub) != -1) {
	    nBindex = sOld.indexOf(sOldSub);

	    sbNew = sbNew.append(sOld.substring(0, nBindex));
	    sbNew = sbNew.append(sNewSub);

	    sOld = sOld.substring(nBindex + nOldLength);

	    sbNew = sbNew.append(sOld);
	    return sbNew.toString();
	} else {
	    return sOld;
	}

    }

    /**
     * 
     * Replace first occurence of a Substring by a new Substring in a String
     * ignoring case.
     * <p>
     * 
     * @param sString
     *            The String to modify.
     * @param sOldChars
     *            The Substring to replace.
     * @param sNewChars
     *            The new Substring to replace with.
     *            <p>
     * @return A String with Substring replacement.
     * 
     */

    public static String replaceFirstIgnoreCase(String sString, String sOldSub,
	    String sNewSub) {

	if (sString == null) {
	    return null;
	}

	StringBuffer sbNew = new StringBuffer();
	String sOld = sString;

	int nOldLength = sOldSub.length();
	int nBindex = 0;

	// if (sOld.indexOf(sOldSub) != - 1)
	if (sOld.toUpperCase().indexOf(sOldSub.toUpperCase()) != -1) {
	    nBindex = sOld.toUpperCase().indexOf(sOldSub.toUpperCase());

	    sbNew = sbNew.append(sOld.substring(0, nBindex));
	    sbNew = sbNew.append(sNewSub);

	    sOld = sOld.substring(nBindex + nOldLength);

	    sbNew = sbNew.append(sOld);
	    return sbNew.toString();
	} else {
	    return sOld.toString();
	}

    }

    /**
     * 
     * Replace all occurences of a Substring by a new Substring in a String
     * <p>
     * 
     * @param sString
     *            The String to modify.
     * @param sOldChars
     *            The Substring to replace.
     * @param sNewChars
     *            The new Substring to replace with.
     *            <p>
     * @return A String with Substring replacement.
     * 
     */

    public static String replaceAll(String sString, String sOldSub,
	    String sNewSub) {

	if (sString == null) {
	    return null;
	}

	StringBuffer sbNew = new StringBuffer();
	String sOld = sString;

	int nOldLength = sOldSub.length();
	int nBindex = 0;

	while (sOld.indexOf(sOldSub) != -1) {
	    nBindex = sOld.indexOf(sOldSub);

	    sbNew = sbNew.append(sOld.substring(0, nBindex));
	    sbNew = sbNew.append(sNewSub);

	    sOld = sOld.substring(nBindex + nOldLength);
	}

	sbNew = sbNew.append(sOld);
	return sbNew.toString();
    }

    /**
     * Replace all occurences of a Substring by a new Substring in a String,
     * ignoring case
     * <p>
     * 
     * @param sString
     *            The String to modify.
     * @param sOldChars
     *            The Substring to replace.
     * @param sNewChars
     *            The new Substring to replace with.
     *            <p>
     * @return A String with Substring replacement.
     * 
     */

    public static String replaceAllIgnoreCase(String sString, String sOldSub,
	    String sNewSub) {

	if (sString == null) {
	    return null;
	}

	StringBuffer sbNew = new StringBuffer();
	String sOld = new String(sString);

	int nOldLength = sOldSub.length();
	int nBindex = 0;

	// while (sOld.indexOf(sOldSub) != - 1)
	while (sOld.toUpperCase().indexOf(sOldSub.toUpperCase()) != -1) {
	    nBindex = sOld.toUpperCase().indexOf(sOldSub.toUpperCase());

	    sbNew = sbNew.append(sOld.substring(0, nBindex));
	    sbNew = sbNew.append(sNewSub);

	    sOld = sOld.substring(nBindex + nOldLength);

	}

	sbNew = sbNew.append(sOld);

	return sbNew.toString();
    }

    /**
     * Count how many times a substring appears in a String
     * 
     * @param sIn
     *            the String
     * @param sSubIn
     *            the substring to search in the String
     * 
     * @return how many times the substring appears in a String
     */

    public static int countSubstring(String sIn, String sSubIn) {
	int nOldLength = sSubIn.length();
	int nBindex = 0;

	int nCount = 0;

	while (sIn.indexOf(sSubIn) != -1) {
	    nBindex = sIn.indexOf(sSubIn);
	    sIn = sIn.substring(nBindex + nOldLength);
	    nCount++;
	}

	return nCount;
    }

    /**
     * 
     * Method to use to *not* display accuentation chars in a String <br>
     * 
     * @param sCleanMe
     *            - String to remove acccents from.
     * 
     * @return String without accentuation chars.
     * 
     */

    public static String removeAccent(String sCleanMe) {
	if (sCleanMe == null) {
	    return null;
	}

	char cCurrent;
	String sReturned = new String();
	for (int i = 0; i < sCleanMe.length(); i++) {
	    cCurrent = sCleanMe.charAt(i);

	    if (!((cCurrent > 'a' && cCurrent < 'z')
		    || (cCurrent > 'A' && cCurrent < 'Z') || Character
			.isDigit(cCurrent))) {
		cCurrent = removeAccent(cCurrent);
	    }

	    sReturned += cCurrent;
	}
	return sReturned;
    }

    /**
     * 
     * Method to use to *not* display accuentation char <br>
     * 
     * @param cCleanMe
     *            - char to remove acccent from.
     * 
     * @return char without accentuation.
     * 
     */

    public static char removeAccent(char cCleanMe) {

	switch (cCleanMe) {
	case '':

	case '':

	case '':
	    return 'a';

	case '':
	    return 'c';

	case '':

	case '':

	case '':

	case '':
	    return 'e';

	case '':

	case '':
	    return 'i';

	case '':
	    return '';

	case '':

	case '':
	    return 'o';

	case '':

	case '':

	case '':
	    return 'u';

	    // UPPERCASE

	case '':

	case '':

	case '':
	    return 'A';

	case '':
	    return 'C';

	case '':

	case '':

	case '':

	case '':
	    return 'E';

	case '':

	case '':
	    return 'I';

	case '':
	    return 'I';

	case '':

	case '':
	    return 'O';

	case '':

	case '':

	case '':
	    return 'U';

	    // other modified chars
	case '':
	    return '\'';

	case '':
	    return '-';

	default:
	    return cCleanMe;
	}
    }

} // End StringMgr
